export interface AddFunctionOptions {
	a: number;
	b: number;
}

export interface AddFunction {
	(options: AddFunctionOptions): number;
}

export class InvalidArgumentTypeError extends TypeError {
	constructor(message: string) {
		super(message);
		this.name = 'InvalidArgumentTypeError';
	}
}

export class ArgumentOutOfRangeError extends RangeError {
	constructor(message: string) {
		super(message);
		this.name = 'ArgumentOutOfRangeError';
	}
}

export const add: AddFunction = (options: AddFunctionOptions): number => {
	const { a, b } = options as unknown as Record<string, unknown>;
	if (typeof a !== 'number' || typeof b !== 'number') {
		throw new InvalidArgumentTypeError('a and b must be numbers');
	}
	if (!Number.isFinite(a) || !Number.isFinite(b)) {
		throw new ArgumentOutOfRangeError('a and b must be finite numbers');
	}

  return a + b;
};
